const debug = require('debug')('response-data-adapter:module')
const ef = require('./entity-fabric')({types: ['adapter', 'definition']})
const jsonValidator = require('./json-validator')
const ResponseDataAdapterException = require('./response-data-adapter-exception')

// схема выходных данных
const defaultAdapterSchema = {
  type: 'object',
  required: ['status'],
  properties: {
    status: {type: ['integer', 'string']},
    response: {type: ['object', 'array', 'number', 'string', 'boolean', 'null']},
    error: {
      type: 'object',
      required: ['message', 'code'],
      properties: {
        code: {type: ['string', 'integer']},
        message: {type: 'string'},
        stacktrace: {type: ['string', 'object']}
      }
    }
  },
  oneOf: [
    {required: ['response']},
    {required: ['error']}
  ]
}

/**
 * @class ResponseDataAdapter
 */
class ResponseDataAdapter {
  constructor (obj = {}) {
    const {
      onError,
      adapterSchema = defaultAdapterSchema
    } = obj
    this.adapterSchema = adapterSchema
    if (onError instanceof Function) {
      debug('onError handler enabled')
      this.onError = onError
    }
  }

  /**
   * @param data
   */
  error (data) {
    if (this.onError instanceof Function) {
      if (data instanceof Error) debug(`error ${data.message}`)
      this.onError(data)
    }
  }
  /**
   * Add new adapter
   *
   * @param id (string) id адаптера, уникальная строка
   * @param adapter (func) колбек
   */
  adapter (id, adapter) {
    if (typeof id === 'string' && id.trim() !== '' && adapter instanceof Function) {
      debug(`add adapter ${id}`)
      const data = adapter({response: {data: {adapterId: id}}, status: 200})
      const jv = jsonValidator().validate(data, this.adapterSchema)
      if (jv.isSuccess()) {
        ef.add('adapter', id, adapter)
        return true
      } else {
        console.error('data after adapter', data)
        debug(`adapter ${id}`, jv.error)
        this.error(new ResponseDataAdapterException('invalid adapter'))
        return false
      }
    }
  }

  /**
   * Add new definition
   *
   * @param id (string) id определения, уникальная строка
   * @param definition (object)
   */
  definition (id, definition) {
    const definitionSchema = {
      type: 'object',
      required: ['adapter'],
      properties: {
        title: {type: 'string'},
        adapter: {type: 'string'},
        schema: {type: 'object'}
      }
    }
    const jv = jsonValidator().validate(definition, definitionSchema)
    if (typeof id === 'string' && id.trim() !== '' && jv.isSuccess()) {
      debug(`add definition ${id}`)
      ef.add('definition', id, definition)
    }
  }

  /**
   * @param obj
   * @returns {boolean}
   */
  exec (obj) {
    const jv = jsonValidator()
    const objSchema = {
      type: 'object',
      required: ['id', 'response', 'status'],
      properties: {
        id: {type: 'string'},
        response: {type: 'object'},
        status: {type: ['string', 'integer']}
      }
    }
    jv.validate(obj, objSchema)
    if (!jv.isSuccess()) {
      this.error(new ResponseDataAdapterException('Wrong data object'))
    } else {
      const {id, response, status} = obj
      const definition = ef.get('definition', id)
      if (definition) {
        const {adapter, schema} = definition.entity
        const adapterObj = ef.get('adapter', adapter)
        if (adapterObj) {
          // если есть схема, проверяем данные
          if (schema) {
            const jv = jsonValidator().validate(response, schema)
            if (!jv.isSuccess()) {
              this.error(new ResponseDataAdapterException('Source response json-schema check is failed'))
              return false
            }
          }

          const data = adapterObj.entity({response, status})
          const jv = jsonValidator().validate(data, this.adapterSchema)
          if (jv.isSuccess()) {
            return data
          } else {
            this.error(new ResponseDataAdapterException('Adaptation data json-schema check is failed'))
            return false
          }
        } else {
          this.error(new ResponseDataAdapterException('Wrong adapter id'))
        }
      } else {
        this.error(new ResponseDataAdapterException(`Wrong definition ${id}`))
      }
    }
  }
}

/**
 * @type {ResponseDataAdapter}
 */
module.exports = ResponseDataAdapter
