const chai = require('chai')
const {expect} = chai
const ResponseDataAdapter = require('../lib/response-data-adapter')
const ResponseDataAdapterException = require('../lib/response-data-adapter-exception')
const rdaModule = require('./../index')

const onError = (err) => {
  if (err instanceof Error) {
    throw err
  }
}

const customSchema = {
  type: 'object',
  required: ['res', 'isError'],
  properties: {
    res: {type: ['object', 'string']},
    isError: {type: 'boolean'}
  }
}
const customAdapter = ({response, status}) => {
  if (status > 200) {
    return {
      isError: true
    }
  } else {
    return {
      isError: false,
      res: (response.hasOwnProperty('data')) ? response.data : String(status)
    }
  }
}

it('module should return instance of class ResponseDataAdapter', () => {
  const rda = rdaModule()
  expect(rda instanceof ResponseDataAdapter).to.equal(true)
})

describe('module should return err', () => {
  const rda = rdaModule({onError})
  it('should return wrong data object', () => {
    try {
      rda.exec({
        id: 'wrongDef'
      })
    } catch (err) {
      console.log(err)
      expect(err instanceof ResponseDataAdapterException).to.equal(true)
      expect(err.message === 'Wrong data object').to.equal(true)
    }
  })

  it('should return wrong definition', () => {
    try {
      rda.exec({
        id: 'wrongDef',
        response: {},
        status: 200
      })
    } catch (err) {
      expect(err instanceof ResponseDataAdapterException).to.equal(true)
      expect(err.message === 'Wrong definition wrongDef').to.equal(true)
    }
  })
})

describe('module should have some methods', () => {
  const rda = rdaModule({onError})
  it('exec should be function', () => {
    expect(rda.exec instanceof Function).to.equal(true)
  })
  it('adapter should be function', () => {
    expect(rda.adapter instanceof Function).to.equal(true)
  })
  it('definition should be function', () => {
    expect(rda.definition instanceof Function).to.equal(true)
  })
})

describe('add adapter', () => {
  const rda = rdaModule({onError, adapterSchema: customSchema})
  const status = rda.adapter('customAdapter', customAdapter)
  it('add adapter should return true', () => {
    expect(status).to.equal(true)
  })

  rda.definition('custom', {
    title: 'definition for test',
    adapter: 'customAdapter'
  })

  it('exec should return right object', () => {
    const data = rda.exec({
      id: 'custom',
      response: {obj: 'some string'},
      status: 200
    })
    expect(data.hasOwnProperty('res')).to.equal(true)
    expect(data.hasOwnProperty('isError') && !data.isError).to.equal(true)
  })
})

describe('add definition', () => {
  const rda = rdaModule({onError, adapterSchema: customSchema})
  rda.adapter('custom', customAdapter)
  const testDefinition = {
    title: 'definition for test',
    adapter: 'custom',
    schema: {
      type: 'object',
      required: ['status'],
      properties: {
        status: {type: 'integer'},
        data: {type: 'object'}
      }
    }
  }
  rda.definition('testDefinition', testDefinition)
  const rightAnswer = rda.exec({
    id: 'testDefinition',
    response: {
      data: {
        foo: 'bar'
      },
      status: 200
    },
    status: 200
  })
  it('should return right object', () =>{
    expect(rightAnswer.hasOwnProperty('res')).to.equal(true)
    expect(rightAnswer.hasOwnProperty('isError') && !rightAnswer.isError).to.equal(true)
  })
})
